var Promise = require('bluebird');
var spawn = require('./spawn-command');
var path = require('path');
var chalk = require('chalk');
var readline = require('readline');

/**
 * install npm package
 * @param {Array} modules
 * @param {String} where
 * @returns {Promise}
 */
module.exports = function install(modules, where) {
  var functionArgs = Array.prototype.slice.call(arguments);
  return new Promise(function(resolve, reject) {
    var args = ['install'].concat(modules).concat('--color=always');

    console.log(chalk.green('正在安装npm包: ' + modules.join(',')));
    
    var child = spawn('tnpm', args, {
      cwd: where
    });

    child.on('error', function(error) {
      var tnpm = spawn('npm', ['list', '--depth=0', '-g', 'tnpm']);

      var output = '';
      tnpm.stdout.on('data', function(data) {
        output += data.toString();
      });

      tnpm.on('close', function() {
        var matches = output.match(/[^\w\d]tnpm@(\d+)/);
        if (matches === null) {
          reject('Please install tnpm (npm install -g tnpm --registry=http://registry.npm.alibaba-inc.com) before continuing.');
        } else if (parseInt(matches[1], 10) < 1) {
          reject('Please update tnpm (tnpm install -g tnpm) before continuing.');
        } else {
          reject('tnpm install error: ' + error);
        }
      });

    });

    var output = '';

    child.stderr.on('data', function(data) {
      output += data.toString();
    }).pipe(process.stderr);

    child.stdout.on('data', function(data) {
      output += data.toString();
    }).pipe(process.stdout);

    child.on('exit', function(code) {
      if (!code) {
        resolve();
        return;
      }

      if (output.match(/'EACCES'/)) {
        var eaccessError = 'Error: EACCES. Please try "[sudo] chown -R $USER ~/.def" to fix the problem.';
        if (process.env.USER && process.env.HOME) {
          var rl = readline.createInterface({
            input: process.stdin,
            output: process.stdout
          });
          rl.question('遇到目录权限问题，是否尝试自动修复 (Y/n)？ ', function(answer) {
            rl.close();
            answer = answer || 'Y';
            if (/y/i.test(answer)) {
              var chown = spawn('sudo', ['chown', '-R', process.env.USER, path.join(process.env.HOME, '.def')], {
                stdio: 'inherit'
              });
              chown.on('close', function() {
                process.nextTick(function() {
                  install.apply(null, functionArgs);
                });
              });
            } else {
              reject(eaccessError);
            }
          });
        } else {
          reject(eaccessError);
        }
      } else {
        reject('tnpm exit with code ' + code);
      }
    });
  });
};